/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <fstream>
#include "ns3/core-module.h"
#include "ns3/simulator-module.h"
#include "ns3/helper-module.h"

using namespace ns3;

NS_LOG_COMPONENT_DEFINE ("EmulatedUdpEchoServerExample");

int 
main (int argc, char *argv[])
{
  //
  // Allow the user to override any of the defaults and the above Bind() at 
  // run-time, via command-line arguments.
  //
  CommandLine cmd;
  cmd.Parse (argc, argv);

  GlobalValue::Bind ("SimulatorImplementationType", 
    StringValue ("ns3::RealtimeSimulatorImpl"));

  NodeContainer n;
  n.Create (1);

  InternetStackHelper internet;
  internet.Install (n);

  EmuHelper emu;
  emu.SetAttribute ("DeviceName", StringValue ("ath0"));
  NetDeviceContainer d = emu.Install (n);

  //
  // We've got the devices in place.  Since we're using MAC address 
  // spoofing under the sheets, we need to make sure that the MAC addresses
  // we have assigned to our devices are unique.  Ns-3 will happily
  // automatically assign the same MAC addresses to the devices in both halves
  // of our two-script pair, so let's go ahead and just manually change them
  // to something we ensure is unique.
  //
  Ptr<NetDevice> nd = d.Get (0);
  Ptr<EmuNetDevice> ed = nd->GetObject<EmuNetDevice> ();
  ed->SetAddress (Mac48Address::ConvertFrom (Mac48Address ("00:00:00:00:00:01")));

  //
  // We've got the "hardware" in place.  Now we need to add IP addresses.
  // This is the server half of a two-script pair.  We need to make sure
  // that the addressing in both of these applications is consistent, so
  // we use provide an initial address in both cases.  Here, the server 
  // will reside on the node with IP address "192.168.1.1"
  //
  Ipv4AddressHelper ipv4;
  NS_LOG_INFO ("Assign IP Addresses.");
  ipv4.SetBase ("192.168.1.0", "255.255.255.0", "0.0.0.1");
  Ipv4InterfaceContainer i = ipv4.Assign (d);

  //
  // Create the UdpEchoServer application on the only node -- node zero.
  // We'll leave it running for a minute.
  //
  NS_LOG_INFO ("Create Applications.");
  UdpEchoServerHelper server (9);
  ApplicationContainer apps = server.Install (n.Get(0));
  apps.Start (Seconds (1.0));
  apps.Stop (Seconds (60.0));

  EmuHelper::EnablePcapAll ("emu-udp-echo-server", false);

  //
  // Now, do the actual simulation.
  //
  NS_LOG_INFO ("Run Simulation.");
  Simulator::Run ();
  Simulator::Destroy ();
  NS_LOG_INFO ("Done.");
}
